# -*- coding: utf-8 -*-

# The maximum length for the fully qualified parameter name is 1011 characters.
# A hierarchy can have a maximum of 15 levels.
# The maximum length for the string value is 4096.

from aws_stash import __project_name__, __version__

import sys

from aws_stash.ParamStore import ParamStore
from aws_stash.Output import Output


def main():
    def valid_path(value):
        if value is None:
            return value
        import re
        pattern = '^/(([A-Za-z0-9-_.]+)/?)*$'
        if not re.match(pattern, value):
            raise argparse.ArgumentTypeError(
                "{0} is an invalid path, must match pattern '{1}'' ".format(value, pattern))
        return value

    import argparse
    import itertools

    parser = argparse.ArgumentParser()

    parser.add_argument('--version', action='version', version='%(prog)s {}'.format(__version__), help='Show version')
    parser.add_argument('path', type=valid_path, help='Path to the parameter key or folder containing parameter keys')
    parser.add_argument('-p', '--params', nargs='+', action='append', default=[], help='Parameter keys')
    parser.add_argument('-w', '--write', nargs='?', const='', help='Write parameter value, leave it empty to input it from STDIN')
    parser.add_argument('-m', '--multi-line', action='store_true', help='Accept multi-line value from STDIN, end input with CTRL+D')
    parser.add_argument('-f', '--force', action='store_true', help='Force overwrite existing value')
    parser.add_argument('-d', '--description', default=None, help='Add a description to the parameter')
    parser.add_argument('-k', '--kms', default='aws/ssm', help='KMS key alias to encrypt the value')
    parser.add_argument('-c', '--copy', action='store_true', help='Copy value to the clipboard instead of showing it')
    parser.add_argument('-o', '--output', choices=['text', 'json', 'export'], default='text', help='Output format')
    parser.add_argument('-l', '--list', action='store_true', help='List all paramaters under same level path')
    parser.add_argument('-r', '--recursive', action='store_true', help='Process all paramaters recursively starting from path')
    parser.add_argument('--delete', action='store_true', help='Delete a single parameter or all parameters recursively starting from path if using --recursive')
    parser.add_argument('-q', '--quiet', action='store_true', help='Output only the values of the parameters')
    parser.add_argument('--full', action='store_true', help='Output fully qualified parameter path')
    parser.add_argument('-v', '--verbose', action='store_true', help='Output parameters details')

    args = parser.parse_args()

    args.params = list(itertools.chain.from_iterable(args.params))

    param_store = ParamStore()
    output = Output()

    if args.write is not None:
        version = param_store.write_parameter(
            path=args.path,
            value=args.write,
            description=args.description,
            force=args.force,
            multi_line=args.multi_line,
            kms=args.kms
        )
        if version is None:
            sys.exit(1)
        if not args.quiet:
            print('Version: {}'.format(version))

    elif args.delete:
        version = param_store.delete_parameters(
            path=args.path,
            recursive=args.recursive
        )

    else:
        if args.list:
            parameters = param_store.list_parameters(
                path=args.path,
                recursive=args.recursive,
                params=args.params,
                verbose=args.verbose
            )
        else:
            parameters = param_store.get_parameters(
                path=args.path,
                params=args.params,
                verbose=args.verbose
            )

        if len(parameters):
            output.output(args, parameters)
        else:
            sys.exit(1)


if __name__ == '__main__':
    main()



from pkg_resources import get_distribution, DistributionNotFound
try:
    __project_name__ = get_distribution(__name__).project_name
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    __project_name__ = __name__
    __version__ = 'unknown'



# -*- coding: utf-8 -*-

import pyperclip


class Output:
    def text(self, options, parameters):
        for parameter in parameters:
            name = parameter['Name'] if options.full or options.list else parameter['Name'].split('/')[-1]
            if options.list:
                print(name)
            elif options.quiet:
                print(parameter['Value'])
            elif options.verbose:
                print('{0}="{1}" version={2} description={3}'.format(name, parameter['Value'], parameter['Version'], parameter.get('Description')))
            else:
                print('{0}="{1}"'.format(name, parameter['Value']))

    def export(self, options, parameters):
        for parameter in parameters:
            name = parameter['Name'] if options.full or options.list else parameter['Name'].split('/')[-1]
            print('export {0}="{1}"'.format(name, parameter['Value']))

    def json(self, options, parameters):
        import json
        output = dict()
        for parameter in parameters:
            name = parameter['Name'] if options.full or options.list else parameter['Name'].split('/')[-1]
            if not options.list:
                if options.verbose:
                    output[name] = {
                        'Value': parameter['Value'],
                        'Version': parameter['Version'],
                        'Description': parameter.get('Description')
                    }
                else:
                    output[name] = parameter['Value']
        if options.list:
            output = list(output.keys())
        elif options.quiet:
            values = list(output.values())
            output = values if len(values) > 1 else values[0]

        print(json.dumps(output, sort_keys=True, indent=2))

    def clipboard(self, options, parameters):
        if len(parameters) == 1:
            pyperclip.copy(parameters[0]['Value'])
            return True
        else:
            return False

    def output(self, options, parameters):
        if options.copy:
            if self.clipboard(options, parameters):
                return
        if options.output == 'text':
            self.text(options, parameters)
        if options.output == 'export':
            self.export(options, parameters)
        elif options.output == 'json':
            self.json(options, parameters)



# coding: utf-8

import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="aws-stash",
    setup_requires=['setuptools_scm'],
    use_scm_version={
        'version_scheme': 'guess-next-dev',
        'local_scheme': 'dirty-tag'
    },
    author="Ivan Lopez",
    author_email="ivan@askai.net",
    description="Manage AWS Parameter Store",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/askainet/aws-stash",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'aws-stash = aws_stash.__main__:main',
        ],
    },
    install_requires=[
        'boto3>=1.7.38',
        'pyperclip==1.6.4'
    ]
)



# -*- coding: utf-8 -*-

# The maximum length for the fully qualified parameter name is 1011 characters.
# A hierarchy can have a maximum of 15 levels.
# The maximum length for the string value is 4096.

import sys
import boto3
from botocore.exceptions import ClientError


class ParamStore:
    def __init__(self):
        self.ssm_client = boto3.client('ssm')

    def get_parameter(self, path, verbose=False):
        try:
            if verbose:

                paginator = self.ssm_client.get_paginator('get_parameter_history')
                page_iterator = paginator.paginate(
                    Name=path,
                    WithDecryption=True
                )
                parameters = list()
                for page in page_iterator:
                    parameters = parameters + page['Parameters']

                if len(parameters):
                    return sorted(parameters, key=lambda k: k['Version'], reverse=True)[0]
                else:
                    return None

            else:

                response = self.ssm_client.get_parameter(
                    Name=path,
                    WithDecryption=True
                )
                return response.get('Parameter')

        except ClientError as e:
            if e.response['Error']['Code'] != 'ParameterNotFound':
                raise e

    def list_parameters(self, path, recursive=False, params=[], verbose=False):

        def add_paths(paths, levels, start_depth, recursive):
            if recursive:
                max_depth = len(levels)
            else:
                max_depth = min(len(levels), start_depth + 1)
            for i in range(start_depth, max_depth):
                path = '/'.join(levels[:i]) + '/'
                if path not in paths:
                    paths.append(path)

        start_depth = len(path.rstrip('/').split('/')) + 1
        parameters = list()
        paths = list()

        paginator = self.ssm_client.get_paginator('get_parameters_by_path')
        page_iterator = paginator.paginate(
            Path=path,
            WithDecryption=True,
            Recursive=True
        )

        for page in page_iterator:
            for parameter in page['Parameters']:
                levels = parameter['Name'].split('/')
                add_paths(paths, levels, start_depth, recursive)
                if not recursive:
                    # Skip parameters in sub-directories when recursive is not enabled
                    if len(levels) > start_depth:
                        continue

                if len(params):
                    print(levels)
                    print(params)
                    if levels[-1] not in params:
                        continue

                parameters.append({'Name': parameter['Name']})

        for path in paths:
            parameters.append({'Name': path})

        parameters = sorted(parameters, key=lambda k: k['Name'])

        if not len(parameters) and not len(params):
            parameter = self.get_parameter(path, verbose)
            if parameter:
                parameters.append(parameter)

        return parameters

    def get_parameters(self, path, params=[], verbose=False):
        parameters = list()

        paginator = self.ssm_client.get_paginator('get_parameters_by_path')
        page_iterator = paginator.paginate(
            Path=path,
            WithDecryption=True
        )

        for page in page_iterator:
            for parameter in page['Parameters']:
                if len(params) and parameter['Name'].split('/')[-1] not in params:
                    continue
                if verbose:
                    parameter = self.get_parameter(parameter['Name'], verbose)
                parameters.append(parameter)

        if not len(parameters) and not len(params):
            parameter = self.get_parameter(path, verbose)
            if parameter:
                parameters.append(parameter)

        return parameters

    def write_parameter(self, path, value='', description=None, force=False, multi_line=False, kms='aws/kms'):
        if value == '':
            for line in sys.stdin:
                value = value + line
                if not multi_line:
                    break
            value = value[:-1]

        if description is None:
            parameter = self.get_parameter(path, verbose=True)
            if parameter:
                description = parameter.get('Description', '')
            else:
                description = ''

        try:
            response = self.ssm_client.put_parameter(
                Name=path,
                Value=value,
                Description=description,
                Type='SecureString',
                KeyId='alias/{}'.format(kms),
                Overwrite=force
            )
        except ClientError as e:
            if e.response['Error']['Code'] == 'ParameterAlreadyExists':
                print('Parameter {0} already exists, use --force if you want to overwrite its value'.format(path))
                return None
            else:
                raise e
        else:
            return response['Version']

    def delete_parameters(self, path, recursive=False):
        if recursive:
            list_params = self.list_parameters(path=path, recursive=recursive)
            parameters = [p['Name'] for p in list_params]
        else:
            parameters = [path]

        if len(parameters):
            response = self.ssm_client.delete_parameters(
                Names=parameters
            )
            print('Deleted parameters: {0}'.format(response['DeletedParameters']))
            print('Invalid parameters: {0}'.format(response['InvalidParameters']))
